unit uPHPSessionFiles;

interface

uses
  SysUtils, Classes, Variants;

type
  //tipos de dados conhecidos pela classe
  TPHPSessionValueType = (vtString, vtBoolean);

type
  TPHPSession = class
  private
    //nome do arquivo que armazena a sesso, preenchido no mtodo Create
    FArqSess: String;
    //StringList utilizada para armazenar as variveis
    Flst: TStringList;

    //mtodo que carrega o arquivo especificado em Flst
    procedure CarregaSessao(APath, ASessID: String);
    //limpa o Flst, eliminando as variveis
    procedure LiberaSessao;

    //retorna a quantidade de variveis existentes na sesso
    function GetVarCount: Integer;
    //localiza uma varivel na sesso, retornando sua posio em Flst
    function SearchVar(AVarName: String): Integer;
    //retorna o valor de uma varivel de uma determinada posio em Flst
    function GetValueFromPosition(APos: Integer;
      AType: TPHPSessionValueType): Variant;
  public
    //propriedade que Conta as variveis, fazendo acesso  funo GetVarCount
    property VarCount: Integer read GetVarCount;

    //testa se uma determinada varivel existe na sesso
    function IsSet(AVarName: String): Boolean;
    //recupera o nome de uma varivel da sesso, dada sua posio
    function GetVarName(AVarIndex: Integer): String;
    //as funes abaixo retornam o valor de uma varivel, porm, uma passando
    //o nome da Varivel e a outra o ndice da mesma. Ambas utilizam o mtodo
    //GetValueFromPosition para recuperar o valor em Flst
    function GetValue(AVarName: String; AType:
      TPHPSessionValueType = vtString): Variant; overload;
    function GetValue(AVarIndex: Integer; AType:
      TPHPSessionValueType = vtString): Variant; overload;

    //construtor da classe, que monta o caminho correto do arquivo de sesso
    //e a carrega em Flst, chamando o mtodo CarregaSessao
    constructor Create(APath, ASessID: String);
    //liebera os objetos criados na classe
    destructor Free;
  end;

implementation

{ TPHPSession }

//mtodo que libera os objetos instanciados nesta classe da memria
destructor TPHPSession.Free;
begin
  LiberaSessao;
  FreeAndNil(Flst);
end;

//limpa o StringList Flst, limpando as variveis da sesso
procedure TPHPSession.LiberaSessao;
begin
  Flst.Clear;
end;

//metdo construtor da classe, que instancia o StringList Flst e
//chama o mtodo CarregaSessao, que preenche este StringList com base
//no arquivo da sesso
constructor TPHPSession.Create(APath, ASessID: String);
begin
  Flst := TStringList.Create;
  CarregaSessao(APath, ASessID);
end;

//Carrega o arquivo de sesso para o stringlist
procedure TPHPSession.CarregaSessao(APath, ASessID: String);
var
  str, tmp, n: String;
  i: Integer;
begin
  //faz as validaes do diretrio e ID da sesso
  if (Trim(APath) = '') and (not DirectoryExists(APath)) then
    raise Exception.Create('Path Inexistente');

  if Trim(ASessID) = '' then
    raise Exception.Create('SessionID Invalido!');

  //monta o caminho absoluto do arquivo de sesso
  //Neste ponto deixei fixo o inicio "sess_" para o nome dos arquivos pois
  //meu servidor est configurado dessa maneira. Porm, em alguns casos no h
  //este prefixo, sendo o nome do arquivo somente o ID da sesso.
  if not (APath[Length(APath)] in ['/','\']) then
    APath := APath + '\';
  FArqSess := APath + 'sess_' + ASessID;

  //valida se o arquivo realmente existe
  if not FileExists(FArqSess) then
    raise Exception.Create('Arquivo de Sessao Nao Encontrado!');  

  //usa o StringList primeiramente para carregar o arquivo temporariamente,
  //para ento passar o contedo para a variavel str, que ser utilizada dentro
  //do while, mais abaixo, que far ento a insero de cada varivel no Flst,
  //fazendo o Copy de acordo com o tipo de dado da varivel
  Flst.LoadFromFile(FArqSess);
  str := Trim(Flst.Text);
  Flst.Clear;
  if Trim(str) = '' then
    exit;

  i := Pos('|', str);
  while i > 0 do
  begin
    tmp := '';
    tmp := Copy(str, 1, i-1);
    if Copy(str,i+1,1) = 's' then
    begin
      tmp := tmp + Copy(str, i, 3);
      Delete(str, 1, i+2);

      n := '';
      while str[1] <> ':' do
      begin
        n := n + str[1];
        Delete(str, 1, 1);
      end;
      Delete(str, 1, 2);

      tmp := tmp + n + ':"' + Copy(str, 1, StrToInt(n)+1 );
      Delete(str, 1, StrToInt(n)+2);
    end
    else if Copy(str,i+1,1) = 'b' then
    begin
      tmp := tmp + Copy(str, i, 4);
      Delete(str, 1, i+4);
    end
    else if Copy(str,i+1,1) = 'N' then
    begin
      tmp := tmp + Copy(str, i, 2);
      Delete(str, 1, i+2);
    end
    else if Copy(str, i+1, 1) = 'i' then
    begin
      tmp := tmp + Copy(str, i, Pos(';', str)-i);
      Delete(str, 1, Length(tmp)+1);
    end;

    i := Pos('|', str);
    Flst.Add(tmp);
  end;
end;

//percorre os itens da StringList Flst, tentando localizar o AVarName
function TPHPSession.SearchVar(AVarName: String): Integer;
var
  c: Integer;
begin
  result := -1;
  for c := 0 to Flst.Count - 1 do
    if (Pos(AVarName, Flst[c]) = 1) and
       (Copy(Flst[c], Length(AVarName)+1, 1) = '|') then
    begin
      result := c;
      break;
    end;
end;

//retorna a quantidade de itens existentes no Flst
function TPHPSession.GetVarCount: Integer;
begin
  result := Flst.Count;
end;

//Retorna o Valor de um varivel com base em seu nome.
function TPHPSession.GetValue(AVarName: String;
  AType: TPHPSessionValueType = vtString): Variant;
var
  x: Integer;
begin
  //Localizar a varivel por Nome e em seguida, solicita seu valor atravs
  // do mtodo GetValueFromPosition
  x := SearchVar(AVarName);
  if x >= 0 then
    result := GetValueFromPosition(x, AType)
  else
    result := 'Varivel ' + AVarName + ' no existente na sesso.';
end;

//Retorna o Valor de um varivel com base em sua posio.
function TPHPSession.GetValue(AVarIndex: Integer;
   AType: TPHPSessionValueType): Variant;
begin
  //valida antes se a posio passada por parmetro  vlida, e sendo,
  //solicita o valor ao mtodo GetValueFromPosition
  if (AVarIndex >= 0) and (AVarIndex < Flst.Count) then
    result := GetValueFromPosition(AVarIndex, AType)
  else
    result := 'Indice ' + IntToStr(AVarIndex) + ' no existente na sesso.';
end;

//faz a identificao do valor da varivel de acordo com o tipo especificado
//por parmetro, que pode ser String ou Boolean. Como cada item do StringList
//pode possuir um formato diferente, so feitos testes com relao ao tipo de
//dado. Vale lembrar que na sesso do php, cada varivel fica no formato
//nome|tipo:tamanho:valor - quando  String, e nome|tipo:valor para os demais,
//podendo o valor ser de 1 ou n posies
function TPHPSession.GetValueFromPosition(APos: Integer;
      AType: TPHPSessionValueType): Variant;
var
  str, n: String;
  i: Integer;
begin
  str := Flst[APos];
  i := Pos('|', str);
  if Copy(str,i+1,1) = 's' then
  begin
    result := '';
    Delete(str, 1, i+2);

    n := '';
    while str[1] <> ':' do
    begin
      n := n + str[1];
      Delete(str, 1, 1);
    end;
    Delete(str, 1, 2);

    result := Trim( Copy(str, 1, StrToInt(n) ) );
    if AType = vtBoolean then
      result := (result = 'S') or (result = 'Y') or
                (result = 's') or (result = 'y') or
                (Length(result) > 0);
  end
  else if Copy(str,i+1,1) = 'b' then
  begin
    result := Copy(str, i+3, 1);
    if AType = vtBoolean then
      result := result = '1';
  end
  else if Copy(str,i+1,1) = 'N' then
  begin
    result := '|indefinido|';
    if AType = vtBoolean then
      result := False;
  end
  else if Copy(str, i+1,1) = 'i' then
    result := Copy(str, i+3, Length(str));
end;

//Testa se uma determinada Varivel est na sesso
//correspondente ao isset($_SESSION['variavel']) do php
function TPHPSession.IsSet(AVarName: String): Boolean;
begin
  result := SearchVar(AVarName) >= 0;
end;

//retorna o nome de uma varivel passada seu ndice na sesso
function TPHPSession.GetVarName(AVarIndex: Integer): String;
begin
  if (AVarIndex >= 0) and (AVarIndex < Flst.Count) then
    result := Copy(Flst[AVarIndex], 1, Pos('|', Flst[AVarIndex])-1)
  else
    result := 'Indice ' + IntToStr(AVarIndex) + ' no existente na sesso.';
end;

end.
