unit ufrmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, MMSystem, uPlayer, ExtCtrls, ComCtrls, ImgList;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    btnOpen: TButton;
    btnPlay: TButton;
    btnStop: TButton;
    Timer1: TTimer;
    ProgressBar1: TProgressBar;
    trkbrLeft: TTrackBar;
    trkbrRight: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    ImageList1: TImageList;
    Label3: TLabel;
    procedure btnPlayClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure trkbrLeftChange(Sender: TObject);
    procedure trkbrRightChange(Sender: TObject);
  private
    Player: TPlayer;
    FWaveStream: TMemoryStream;

    procedure DataRequired(sender: TObject; const Buffer: pointer;
      const BufferSize: cardinal; var BytesInBuffer: cardinal);
    procedure LoadFormat(FMem: TMemoryStream; var FWaveFormatEx: TWaveFormatEx);
    { Private declarations }
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

procedure TForm1.DataRequired(sender:TObject;
                            const Buffer:pointer;
                            const BufferSize:cardinal;
                            var BytesInBuffer:cardinal);
begin
  if FWaveStream.Position+BufferSize<FWaveStream.Size then
  begin
    BytesInBuffer := BufferSize;
    FWaveStream.ReadBuffer(Buffer^,BufferSize);
  end else
  begin
    BytesInBuffer := FWaveStream.Size-FWaveStream.Position;
    FWaveStream.ReadBuffer(Buffer^,BytesInBuffer);
  end;
end;

constructor TForm1.Create(AOwner: TComponent);
begin
  inherited;
  FWaveStream:=TMemoryStream.Create;

  Player := TPlayer.Create;
  Player.OnDataRequired:=DataRequired;
end;

destructor TForm1.Destroy;
begin
  Player.Free;
  FWaveStream.Free;
  inherited;
end;

procedure TForm1.btnPlayClick(Sender: TObject);
var dataOffset:integer;
begin

  dataOffset:=4+ SizeOf(DWORD)+
            4 + 4 +SizeOf(DWORD)+
           SizeOf(TWaveFormatEx) + 4 + SizeOf(DWORD);
  FWaveStream.Seek(dataOffset,soFromBeginning);

  ProgressBar1.Max:=FWaveStream.Size-dataOffset;
  ProgressBar1.Min:=0;
  ProgressBar1.Position:=0;

  Player.Start;

  btnStop.Enabled := True;
  btnPlay.Enabled := False;
  Timer1.Enabled := True;
end;

procedure TForm1.btnStopClick(Sender: TObject);
begin
  Player.Stop;
  ProgressBar1.Position := 0;
  btnPlay.Enabled := True;
  btnStop.Enabled := False;
end;

procedure TForm1.LoadFormat(FMem: TMemoryStream;
                       var FWaveFormatEx:TWaveFormatEx);
var id:array[0..3] of char;
    len:integer;
begin
    FMem.Seek(0,soFromBeginning);
    FMem.ReadBuffer(id[0],4);
    if (id='RIFF') then
    begin
      FMem.Seek(4,soFromCurrent);
      FMem.ReadBuffer(id[0],4);
      if (id='WAVE') then
      begin
        FMem.ReadBuffer(id[0],4);
        if (id='fmt ') then
        begin
          FMem.ReadBuffer(len,4);
          if (len=Sizeof(TWaveFormatEx)) then
          begin
            FMem.ReadBuffer(FWaveFormatEx,len);
          end else
          if (len=Sizeof(TPCMWaveFormat)) then
          begin
            FMem.ReadBuffer(FWaveFormatEx,len);
            FWaveFormatEx.cbSize:=0;
          end else
          begin
            FMem.Clear;
            raise Exception.Create('Formato de WAV no suportado.');
          end;
        end else
        begin
          FMem.Clear;
          raise Exception.Create('Formato de WAV invlido.');
        end;
      end else
      begin
        FMem.Clear;
        raise Exception.Create('Formato de WAV desconhecido.');
      end;
    end else
    begin
      FMem.Clear;
      raise Exception.Create('Formato de WAV desconhecido.');
    end;
end;

procedure TForm1.btnOpenClick(Sender: TObject);
var afile:TFileStream;
    awaveFormat:TWaveFormatEx;
begin
  Player.Stop;
  if OpenDialog1.Execute then
  begin
    afile:=TFileStream.Create(OpenDialog1.FileName,fmOpenRead);
    try
      FWaveStream.Clear;
      FWaveStream.CopyFrom(afile,0);
      LoadFormat(FWaveStream,awaveFormat);
      Player.Channel:=awaveFormat.nChannels;
      Player.SamplePerSec:=awaveFormat.nSamplesPerSec;
      Player.BitsPerSample:=awaveFormat.wBitsPerSample;

      Player.Open;

      trkbrLeft.Position:=Player.LeftVolume;
      trkbrRight.Position:=Player.RightVolume;

      trkbrLeft.enabled:=true;
      trkbrRight.enabled:=true;
      btnPlay.Enabled:=true;
    finally
      afile.Free;
    end;
  end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  if ProgressBar1.Position=ProgressBar1.Max then
    begin
    btnPlay.Enabled := True;
    btnStop.Enabled := False;
    Timer1.Enabled := False;
    ProgressBar1.Position := 0;
    Player.Stop;
    end
  else
    ProgressBar1.Position:=Player.CurrentPosBytes;
end;

procedure TForm1.trkbrLeftChange(Sender: TObject);
begin
  Player.LeftVolume := trkbrLeft.Position;
end;

procedure TForm1.trkbrRightChange(Sender: TObject);
begin
  Player.RightVolume := trkbrRight.Position;
end;

end.
